<?php

/**
 * openMosixWebView - produces web charts for monitoring an openMosix cluster
 *
 * Copyright (C) 2002-2003  Ramon Pons Vivanco  <rpons@laurel.datsi.fi.upm.es>
 *                                              <rpons@rinu.org>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA.
 */

/* $Id: openmosixwebview.php,v 1.47 2003/02/16 11:11:57 rpons Exp $ */

/**
 * openMosixWebView produces web charts for monitoring an openMosix cluster.
 *
 * openMosixWebView reads openMosixcollector's logs and /proc/hpc
 * 
 */

/* Global variables */
$host = "madreselva"; /* The name of the openMosix cluster */
$clusterlogdir = "/tmp/openmosixcollector/"; /* Logdir */
$dirphist = $clusterlogdir . "phist/"; /* Process history */
$tsecsdat = "tsecs.dat"; /* phist extension */
$procdir = "/proc/mosix/nodes/"; /* HPC info */
$adminconfig = "/proc/mosix/admin/config"; /* We use it for whois info */
$group = "/etc/group"; /* group's file */
$defaultwidth = 300; /* Default width */
$defaultheight = 200; /* Default height */
$defaultnode = 1; /* If no arg n=X is supplied, we use this on graph=node */
$units=1048576; /* MB = 1024 * 1024 */
$barwidth="500"; /* The width of bar.png */

if (!$ninc){

	require('chart.php');

	/* Returns an array. Each array's entry is one line of the file */
	function getfile ($f){
		$linesarray=@file($f);
		return $linesarray;
	}

	/* Returns the number of cpu's of the cluster */
	function ncpus () {
		global $procdir;
		$n=-2; /* We don't want "." and ".." */
		if ($dir = @opendir($procdir)) {
			while (readdir($dir) !== false) $n++;  
			closedir($dir);
		}
		if ($n < 0) $n=0;
		return $n;
	}

	/* Returns an array with node's id of the cluster */
	function cids () {
		global $procdir;
		$i=0;
		if ($dir = @opendir($procdir)){
			while (($n = readdir($dir)) !== false)
				if (is_numeric($n))
					$arrayCI[$i++]=$n;
			closedir($dir);
		}
		return $arrayCI;
	}

	/* Parse /tmp/openmosixcollector */
	function parsecollector($conf){
		global $clusterlogdir;
		/* Choose file to read */
		if ($conf->graph=="node") $linesarray = getfile($clusterlogdir . $conf->n);
		else $linesarray = getfile($clusterlogdir . "cluster");

		/*
		 * Line to parse:
		 * 13.11.2002-20.52.50 0 200 5376 22 6
		 *        T            L  B   M   U  C
		 * 
		 *        T -> currenttime
		 *        L -> load
		 *        B -> balancing (200-diff)
		 *        M -> available memory
		 *        U -> used memory
		 *        C -> cpus
		 */
		
		$i=0;
		/* Complexity: O(N) */
		if (is_array($linesarray)){
			foreach ($linesarray as $line){
				list($T, $L, $B, $M, $U, $C) = sscanf($line,"%s %d %d %d %d %d");
				$conf->arrayL[$i]=$L;
				$conf->arrayB[$i]=($B - 100);
				$conf->arrayM[$i]=$M;
				$conf->arrayU[$i]=$U;
				$conf->arrayC[$i]=$C;
				$conf->arrayUM[$i]= ($U * $M)/100;
				$i++;
			}
		} else {
			$conf->down=1;
		}
		return $conf;
	}

	/* Get openMosix information from /proc */
	function openmosix ($node, $val){
		global $procdir; global $adminconfig;

		/* We don't have whois information directly in /proc */
		if ($val != "whois"){
			$file = $procdir . $node . "/" . $val;
			$fp = fopen ($file, "r");
			$line =fgets ($fp, 1024);
			fclose($fp);
			sscanf($line, "%d", &$out);
			if ($out < 0) $out=0;
		}

		switch ($val){
		case "status":
			sscanf($line, "%d", &$out);
			if ($out == 3) 
				$out = 1;
			else
				$out = 0;
			break;
		case "whois":
			/*	$out=`mosctl whois $node`; */
			$openmosixnetLength = 24; // size in bytes of struct mosixnet
			$i=0;

			$fp=fopen($adminconfig, 'rb');

			/**
			 * The replacement of `mosctl whois $node`
			 *  is powered by: Mirko Caserta <mirko@mcaserta.com>
			 */
			$tmpByteString = fread($fp, $openmosixnetLength);
			do { // stop when we found the requested IP or EOF or $tmpByteString==0
				$tmp = unpack('InodeNumber/vaf/vunused/C4ip/I2unused2/Icpus', $tmpByteString);
				if ($tmp["nodeNumber"] == $node){
					$out = $tmp["ip1"] . '.' . $tmp["ip2"] . '.' . $tmp["ip3"] . '.' . $tmp["ip4"];
				} else {
					$pos = $i++ * $openmosixnetLength;
					fseek($fp, $pos);
					$tmpByteString = fread($fp, $openmosixnetLength);
				}
			} while ($tmp["nodeNumber"] != $node && !feof($fp) && strlen($tmpByteString));
			fclose($fp);
			break;
		}
		return $out;
	}

	/* Returns the number of cpu's of node i */
	/* (Now not used) */
	function cpusnode ($i) {
		global $procdir;
		$a=@file($procdir . $i . "/cpus");
		sscanf($a[0], "%d", &$n);
		return $n;
	}

	/* Returns an array with node's load */
	function nodesload ($conf) {
		global $conf;
		$i=0;
		$conf->maxload=0.0;
		/* $arrayL = [0 ..... 0] */
		$arrayL[$i]=0;
		foreach($conf->ci as $n){
			$l = openmosix($n, "load");
			$l = ($l / 100);
			$arrayL[++$i]=$l;
			if ($l>$conf->maxload) $conf->maxload=$l;  /* Set the array's max */
		}
		$arrayL[++$i]=0;
		return $arrayL;
	}

	/* Returns an array with node's memory */
	function nodesmem ($conf) {
		global $conf;
		global $units;
		$i=0;
		$conf->maxmem=0;
		$mem->used[$i]=0;
		$mem->total[$i]=0;
		foreach($conf->ci as $n){
			$t = openmosix($n, "tmem");
			$m = openmosix($n, "mem");
			$mem->used[++$i]= ($t - $m) / $units;
			$mem->total[$i]= $t / $units;
			if ($t > $conf->maxmem) $conf->maxmem = $t;  /* Set the array's max */
		}
		$conf->maxmem = $conf->maxmem / $units;
		$mem->used[++$i]=0;
		$mem->total[$i]=0;
		return $mem;
	}

	/* Returns an array with node's speed */
	function nodesspeed ($conf) {
		global $conf;
		$i=0;
		$conf->maxspeed=0;
		/* $arrayS = [0 ..... 0] */
		$arrayS[$i]=0;
		foreach($conf->ci as $n){
			$s=openmosix($n, "speed");
			$arrayS[++$i]=$s;
			if ($s>$conf->maxspeed) $conf->maxspeed=$s;  /* Set the array's max */
		}
		$arrayS[++$i]=0;
		return $arrayS;
	}

	/* Returns an array with node's utilizability percentage */
	function nodesutil ($conf) {
		global $conf;
		$i=0;
		$conf->maxutil=0;
		/* $arrayU = [0 ..... 0] */
		$arrayU[$i]=0;
		foreach($conf->ci as $n){
			$u=openmosix($n, "util");
			$arrayU[++$i]=$u;
			if ($u>$conf->maxutil) $conf->maxutil=$u;  /* Set the array's max */
		}
		$arrayU[++$i]=0;
		return $arrayU;
	}

	function monitor ($conf){
		$i=0;
		foreach($conf->ci as $n){
			$conf->arrayMO["node"][$i]=$n;

			$s = openmosix($n, "status");
			$conf->arrayMO["status"][$i]=$s;

			$w = openmosix($n, "whois");
			$conf->arrayMO["whois"][$i]=$w;

			$c = openmosix($n, "cpus");
			$conf->arrayMO["cpus"][$i]=$c;
		
			$l = openmosix($n, "load");
			$conf->arrayMO["load"][$i]=$l;

			$sp = openmosix($n, "speed");
			$conf->arrayMO["speed"][$i]=$sp;

			$um = openmosix($n, "mem");
			$conf->arrayMO["umem"][$i]=$um;

			$tm = openmosix($n, "tmem");
			$conf->arrayMO["tmem"][$i]=$tm;

			$uf = openmosix($n, "rmem");
			$conf->arrayMO["ufree"][$i]=$uf;

			$u = openmosix($n, "util");
			$conf->arrayMO["util"][$i]=$u;

			$i++;
		}
		return $conf;
	}

	/* Parse /proc/#pid/stat file */
	function parsestat($pid){
		
		$fp=fopen("/proc/" . $pid . "/stat", "r");
		$line=fgets($fp, 4096); // Read the first line
		/* god exists ? */
		list($P->pid, $P->cmd, $P->state, $P->ppid, $P->pgrp,
			  $P->session, $P->tty, $P->tpgid, $P->flags,
			  $P->min_flt, $P->cmin_flt, $P->maj_flt,
			  $P->cmaj_flt, $P->utime, $P->stime, $P->cutime,
			  $P->cstime, $P->priority, $P->nice, $P->timeout,
			  $P->it_real_value, $P->start_time, $P->vsize,
			  $P->rss, $P->rss_rlim, $P->start_code, $P->end_code,
			  $P->start_stack, $P->kstk_esp, $P->kstk_eip,
			  $P->wchan, $P->nswap, $P->cnswap, $P->exit_signal,
			  $P->processor) = sscanf($line,
											  "%d (%s %c %d %d %d %d %d %lu %lu %lu %lu %lu %lu %lu %ld %ld %ld %ld %ld %ld %lu %lu %ld %lu %lu %lu %lu %lu %lu %*s %*s %*s %*s %lu %lu %lu %d %d");
		$P->cmd=substr($P->cmd, 0, strlen($P->cmd)-1); // remove the final ) of the cmdline
		fclose($fp);
		return $P;
	}

	/* Get information about the process running on the system */
	function procs($conf) {
		$i=0;

		/* Get pids */
		if ($dir = @opendir("/proc")) {
			while (($file = readdir($dir)) !== false) {
				if (is_numeric($file)){
					$tmppid[$i++]=$file;
				}
			}
			closedir($dir);
		}

		/* Get current time0 */
		list($usec, $sec) = explode(" ", microtime()); 
		$time0=(float)$usec + (float)$sec;
		
		/* Get utime+stime time for each pid */
		$i=0;
		foreach ($tmppid as $pid){
			/* Parse /proc/#/stat */
			$P=parsestat($pid);
			$tmp["total_time"][$i]=$P->utime + $P->stime;
			$conf->arrayP["nice"][$i++]=$P->nice;
		}

		/* Get proc values */
		$i=0;
		foreach ($tmppid as $pid){
			$a=@file("/proc/" . $pid . "/status");
		
			/*	$conf->arrayP["pid"][$i]=$pid; */
			// We set this in the last loop to
			// avoid showing dead process

			sscanf($a[0],"Name: %s", &$n);
			$conf->arrayP["cmdline"][$i]=$n;
			
			$a=@file("/proc/" . $pid . "/nmigs");
			sscanf($a[0],"%d", &$nmigs);
			$conf->arrayP["nmigs"][$i]=$nmigs;
			
			$a=@file("/proc/" . $pid . "/where");
			sscanf($a[0],"%d", &$n);
			$conf->arrayP["n"][$i]=$n;
			
			$a=@file("/proc/" . $pid . "/lock");
			sscanf($a[0],"%d", &$lock);
			$conf->arrayP["lock"][$i]=$lock;

			sscanf($a[1],"State: %c", &$n);
			$conf->arrayP["stat"][$i]=$n;
			
			sscanf($a[6],"Uid: %d", &$n);
			$conf->arrayP["uid"][$i++]=$n;
		}

		/* Sleep to have a bigger $time1 */
		usleep(600000);

		/* Get current time1 */
		list($usec, $sec) = explode(" ", microtime()); 
		$time1=(float)$usec + (float)$sec;

		/* time1 - time0 */
		$time=($time1 - $time0)*1000.0;

		/* Read again utime+stime and calculate delta */
		$i=0;
		foreach ($tmppid as $pid){
			/* Parse /proc/#/stat */
			$P=parsestat($pid);
			$ttime=($P->utime + $P->stime) - $tmp["total_time"][$i];
			$conf->arrayP["pid"][$i]=$pid;

			$cpu = ((float)$ttime * 999.9) / (float)$time;
			if ($cpu > 99.9) $cpu = 99.9;
			$conf->arrayP["cpu"][$i++]=$cpu;
		}

		return $conf;
	}

	/* Gets the history of the processes */
	function parsephist($conf) {
		global $x;
		$tmax=1; $i=0;
		$conf->selected=1; /* default file 1tsecs.dat */

		/* find requested file */
		if ($conf->time && ($dir = @opendir($conf->dirphist))) {
			while ((($file = readdir($dir)) !== false)) { /* need to read all the files because we need tmax */
				sscanf($file, "%d" . $conf->tsecsdat, $a);
				if ($a){ /* avoid "." and ".." */
					$fp = fopen ( $conf->dirphist . $a . $conf->tsecsdat, "r");
					$firstline=fgets ($fp, 1024);
					fclose($fp);
					sscanf($firstline,"%s", &$t); /* Get timestamp from file */
					if (!$conf->timestamp){
						$conf->timestamp=$t; /* Get first timestamp */
						$conf->selected=$a;
					}
					if ($t==$conf->time){ /* Fuck me plenty */
						$conf->selected=$a;
						$conf->timestamp=$t;
						$x=$a;
					}
					if ($a > $tmax) $tmax=$a;
				}
			}
			$x=intval((($x+1) * $conf->barwidth)/ $tmax);
			closedir($dir);
		} elseif ($conf->barorder){ /* Horizontal scroll bar stuff */
			/* Get $tmax */
			if ($dir = @opendir($conf->dirphist)) {
				while ((($file = readdir($dir)) !== false)) {
					sscanf($file, "%d" . $conf->tsecsdat, $a);
					if ($a){ /* avoid "." and ".." */
						if ($a > $tmax) $tmax=$a;
					}
				}
				closedir($dir);
			}
			$a=intval((($x+1) * $tmax) / $conf->barwidth);
			if ($a > $tmax) $a=$tmax;
			if ($a < 1) $a=1;
			$fp = fopen ( $conf->dirphist . $a . $conf->tsecsdat, "r");
			$firstline=fgets ($fp, 1024);
			fclose($fp);
			sscanf($firstline,"%s", &$t); /* Get timestamp from file */
			$conf->timestamp=$t;
			$conf->selected=$a;
		} else { /* $conf->time is not set */
			if ($dir = @opendir($conf->dirphist)){
				$file = readdir($dir); /* get .  */
				$file = readdir($dir); /* get .. */
				$file = readdir($dir); /* get random file */
				closedir($dir);
				sscanf($file, "%d" . $conf->tsecsdat, &$a);
				$fp = fopen ( $conf->dirphist . $a . $conf->tsecsdat, "r");
				$firstline=fgets ($fp, 1024);
				fclose($fp);
				sscanf($firstline,"%s", &$t); /* Get timestamp */
				$conf->timestamp=$t;
				$conf->selected=$a;
				$x=1;
			}
		}
		$filearray=@file($conf->dirphist . $conf->selected . $conf->tsecsdat);

		/*
		 * 20.11.2002-16.46.23 649 0 0 0 S monkey 0 33 /usr/sbin/apache
		 *          T           P  W L M S   O    N  U        C
		 *
		 *  T -> timestamp
		 *  P -> PID of the process
		 *  W -> on which node the process is running
		 *  L -> if the process is locked
		 *  M -> how often the process migrates
		 *  S -> status
		 *  O -> cannot move the process
		 *  N -> nice (process priority)
		 *  U -> userid
		 *  C -> commandline
		 */
		foreach ($filearray as $line){
			list($T, $P, $W, $L, $M, $S, $O, $N, $U, $C) = sscanf($line,"%s %d %d %d %d %s %s %d %d %s");
			$conf->arrayPH["pid"][$i]=$P;
			$conf->arrayPH["n"][$i]=$W;
			$conf->arrayPH["lock"][$i]=$L;
			$conf->arrayPH["nmigs"][$i]=$M;
			$conf->arrayPH["stat"][$i]=$S;
			$conf->arrayPH["cantmove"][$i]=$O;
			$conf->arrayPH["nice"][$i]=$N;
			$conf->arrayPH["uid"][$i]=$U; ;
			$conf->arrayPH["cmdline"][$i]=$C;
			$i++;
		}
		return $conf;
	}

	function remote ($conf){
		$i=0;
		if ($dir = @opendir("/proc/mosix/remote")) {
			while (($file = readdir($dir)) !== false) {
				if (is_numeric($file)){
					$target= "/proc/mosix/remote/" . $file . "/";

					$line=file($target . "from");
					list($node, $ip) = sscanf($line[0],"%d %s");
					$conf->arrayREMOTE["node"][$i]=$node;
					$conf->arrayREMOTE["ip"][$i]=$ip;

					$line=file($target . "identity");
					sscanf($line[0],"pid=%d",&$pid);
					$conf->arrayREMOTE["pid"][$i]=$pid;
					$i++;
				}
			}
			closedir($dir);
		}
		return $conf;
	}

	/******************/
	/* Plot functions */
	/******************/

	/* This function shows the chart "Load - Balancing" */
	function balplot ($conf){
		$chart = new chart($conf->width, $conf->height);
		$chart->set_title($conf->title);
		$chart->set_labels("","Percent");
		if (!$conf->down)	$chart->set_extrema(1);
		$chart->set_margins (40,10,20,38);
		$chart->plot($conf->bluearray, false, "blue");
		$chart->plot($conf->redarray, false, "red");
		$chart->add_legend($conf->bluelegend, "blue");
		$chart->add_legend($conf->redlegend, "red");
		$chart->stroke();
	}

	/* This function shows the chart "Memory (total VS used)" */
	function avaplot ($conf){
		$chart = new chart($conf->width, $conf->height);
		$chart->set_title($conf->title);
		$chart->set_labels("","MB");
		if (!$conf->down)	$chart->set_extrema(1);
		$chart->set_margins (49,10,20,38);
		$chart->plot($conf->bluearray, false, "blue");
		$chart->plot($conf->redarray, false, "red");
		$chart->add_legend($conf->bluelegend, "blue");
		$chart->add_legend($conf->redlegend, "red");
		$chart->stroke();
	}

	/* This function shows a chart with node's load */
	function nodeplot ($conf){
		$chart = new chart($conf->width, $conf->height);
		$chart->set_title($conf->title);
		$chart->set_labels("","Percent");
		$chart->set_margins (35,10,20,38);
		$chart->plot($conf->redarray, false, "red");
		$chart->add_legend($conf->redlegend, "red");
		$chart->plot($conf->bluearray, false, "blue");
		$chart->add_legend($conf->bluelegend, "blue");
		$chart->stroke();
	}

	/* This function shows a chart with the load of each node */
	function loadplot ($conf){
		$conf->ncpus=count($conf->redarray) - 2; /* Get the number of cpus */
		if ($conf->ncpus < 0) $conf->ncpus=0;
		/* $conf->x_axis_labels = [0 1 2 .... n+1 0] */
		$conf->x_axis_labels = array_merge(array(0), $conf->ci, array(0));
		/* $conf->x_axis_labels is an array to set labels on the X axis */

		$chart = new chart($conf->width, $conf->height);
		$chart->set_title($conf->title);
		$chart->set_labels("Total " . $conf->ncpus . " CPUs", "load");
		$chart->set_extrema(0, $conf->maxload, 0, $conf->ncpus + 1);
		$chart->set_margins (45,10,20,38);
		$chart->set_x_ticks ($conf->x_axis_labels, "text");
		$chart->plot($conf->redarray, false, "red", "impulse");
		$chart->add_legend($conf->redlegend, "red");
		$chart->stroke();
	}

	/* This function shows the chart for "Memory" */
	function memplot ($conf){
		$conf->ncpus=count($conf->redarray) - 2; /* Get the number of cpus */
		if ($conf->ncpus < 0) $conf->ncpus=0;
		/* $conf->x_axis_labels = [0 1 2 .... n+1 0] */
		$conf->x_axis_labels = array_merge(array(0), $conf->ci, array(0));
		/* $conf->x_axis_labels is an array to set labels on the X axis */

		$chart = new chart($conf->width, $conf->height);
		$chart->set_title($conf->title);
		$chart->set_labels("                 Total " . $conf->ncpus . " CPUs", "MB");
		$chart->set_extrema(0, $conf->maxmem, 0, $conf->ncpus + 1);
		$chart->set_margins (40,10,20,38);
		$chart->set_x_ticks ($conf->x_axis_labels, "text");
		$chart->plot($conf->redarray, false, "blue", "impulse");
		$chart->add_legend($conf->redlegend, "red");
		$chart->plot($conf->bluearray, false, "red", "impulse");
		$chart->add_legend($conf->bluelegend, "blue");
		$chart->stroke();
	}

	/* This function shows a chart with the speed of each node */
	function speedplot ($conf){
		$conf->ncpus=count($conf->redarray) - 2; /* Get the number of cpus */
		if ($conf->ncpus < 0) $conf->ncpus=0;
		/* $conf->x_axis_labels = [0 1 2 .... n+1 0] */
		$conf->x_axis_labels = array_merge(array(0), $conf->ci, array(0));
		/* $conf->x_axis_labels is an array to set labels on the X axis */

		$chart = new chart($conf->width, $conf->height);
		$chart->set_title($conf->title);
		$chart->set_labels("Total " . $conf->ncpus . " CPUs","speed");
		$chart->set_extrema(0, $conf->maxspeed, 0, $conf->ncpus + 1);
		$chart->set_margins (47,10,20,38);
		$chart->set_x_ticks ($conf->x_axis_labels, "text");
		$chart->plot($conf->redarray, false, "red", "impulse");
		$chart->add_legend($conf->redlegend, "red");
		$chart->stroke();
	}

	/* This function shows a chart with the load of each node */
	function utilplot ($conf){
		$conf->ncpus=count($conf->redarray) - 2; /* Get the number of cpus */
		if ($conf->ncpus < 0) $conf->ncpus=0;
		/* $conf->x_axis_labels = [0 1 2 .... n+1 0] */
		$conf->x_axis_labels = array_merge(array(0), $conf->ci, array(0));
		/* $conf->x_axis_labels is an array to set labels on the X axis */

		$chart = new chart($conf->width, $conf->height);
		$chart->set_title($conf->title);
		$chart->set_labels("         Total " . $conf->ncpus . " CPUs", "Percent");
		$chart->set_extrema(0, $conf->maxutil+1, 0, $conf->ncpus + 1);
		$chart->set_margins (40,10,20,38);
		$chart->set_x_ticks ($conf->x_axis_labels, "text");
		$chart->plot($conf->redarray, false, "red", "impulse");
		$chart->add_legend($conf->redlegend, "red");
		$chart->stroke();
	}

	/* Return a well formed <td> tag */
	function show ($text, $class=""){
		$web .= "\n\t<td class=\"$class\">";
		$web .= $text . "</td>";
		return $web;
	}

	/* Returns a well formed link */
	function putlink($conf, $i, $g){
		$text=$i;
		if ($i=="cpu") 
			$text = "%cpu";
		return "<a href=\"openmosixwebview.php?graph=$g&amp;sort=$i$t\"><b>$text</b></a>";
	}

	/* Returns the generic web head */
	function webhead($refresh="0", $title="openMosixWebView"){
		global $graph;
			$web = "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\">\n<html>\n<head>\n";
			if ($refresh)
				$web .= " <meta http-equiv=\"Refresh\" content=\"$refresh;url=openmosixwebview.php?graph=$graph\">\n";
			$web .= " <title>$title</title>\n";
			$web .= " <link rel=\"stylesheet\" href=\"style.css\">\n";
			$web .= "</head>\n";
			$web .= "<body>\n";
			return $web;
	}

	/* Shows the web for graph=monitor */
	function monitorplot ($conf){
		global $units, $sort, $des, $csort, $order;
		$i=0;

		/* web head */
		if ($conf->doweb != "no"){

			if ($order)
				$des=!$des;
			
			setcookie ("des", $des);
		
			if ($sort)
				setcookie ("csort", $sort);
			else
				$sort=$csort;

			/* web head */
			$conf->web .= webhead(10);
		}
		/* Table */
		$conf->web .= "<center>\n";
		$conf->web .= "<table class=\"m\">\n";

		$conf->web .= "<tr>";

		$conf->web .= show (putlink($conf, "node", "monitor"), "lr");
		$conf->web .= show (putlink($conf, "status", "monitor"), "dr");
		$conf->web .= show (putlink($conf, "whois", "monitor"), "lr");
		$conf->web .= show (putlink($conf, "cpus", "monitor"), "dr");
		$conf->web .= show (putlink($conf, "load", "monitor"), "lr");
		$conf->web .= show (putlink($conf, "speed", "monitor"), "dr");
		$conf->web .= show (putlink($conf, "umem", "monitor"), "lr");
		$conf->web .= show (putlink($conf, "ufree", "monitor"), "dr");
		$conf->web .= show (putlink($conf, "tmem", "monitor"), "lr");
		$conf->web .= show (putlink($conf, "util", "monitor"), "dr");
		$conf->web .= "\n</tr>\n";
		if (!$conf->down){
			/* Sort */
			if ($des)
				$sortflag = SORT_DESC;
			else
				$sortflag = SORT_ASC;

			switch ($sort){ /* Begin Porn */
			case "node":
				array_multisort($conf->arrayMO ["node"], SORT_NUMERIC, $sortflag,
									 $conf->arrayMO ["status"], $conf->arrayMO ["whois"],
									 $conf->arrayMO ["cpus"], $conf->arrayMO ["load"],
									 $conf->arrayMO ["speed"], $conf->arrayMO ["umem"],
									 $conf->arrayMO ["ufree"], $conf->arrayMO ["tmem"],
									 $conf->arrayMO ["util"]);
				break;
			case "status":
				array_multisort($conf->arrayMO ["status"], SORT_STRING, $sortflag,
									 $conf->arrayMO ["node"], $conf->arrayMO ["whois"],
									 $conf->arrayMO ["cpus"], $conf->arrayMO ["load"],
									 $conf->arrayMO ["speed"], $conf->arrayMO ["umem"],
									 $conf->arrayMO ["ufree"], $conf->arrayMO ["tmem"],
									 $conf->arrayMO ["util"]);
				break;
			case "whois":
				array_multisort($conf->arrayMO ["whois"], SORT_STRING, $sortflag,
									 $conf->arrayMO ["node"], $conf->arrayMO ["status"],
									 $conf->arrayMO ["cpus"], $conf->arrayMO ["load"],
									 $conf->arrayMO ["speed"], $conf->arrayMO ["umem"],
									 $conf->arrayMO ["ufree"], $conf->arrayMO ["tmem"],
									 $conf->arrayMO ["util"]);
				break;
				
			case "cpus":
				array_multisort($conf->arrayMO ["cpus"], SORT_NUMERIC, $sortflag,
									 $conf->arrayMO ["node"], $conf->arrayMO ["status"],
									 $conf->arrayMO ["whois"], $conf->arrayMO ["load"],
									 $conf->arrayMO ["speed"], $conf->arrayMO ["umem"],
									 $conf->arrayMO ["ufree"], $conf->arrayMO ["tmem"],
									 $conf->arrayMO ["util"]);
				
				break;
			case "load":
				array_multisort($conf->arrayMO ["load"], SORT_NUMERIC, $sortflag,
									 $conf->arrayMO ["node"], $conf->arrayMO ["status"],
									 $conf->arrayMO ["whois"], $conf->arrayMO ["cpus"],
									 $conf->arrayMO ["speed"], $conf->arrayMO ["umem"],
									 $conf->arrayMO ["ufree"], $conf->arrayMO ["tmem"],
									 $conf->arrayMO ["util"]);
				
				break;
			case "speed":
				array_multisort($conf->arrayMO ["speed"], SORT_NUMERIC, $sortflag,
									 $conf->arrayMO ["node"], $conf->arrayMO ["status"],
									 $conf->arrayMO ["whois"], $conf->arrayMO ["cpus"],
									 $conf->arrayMO ["load"], $conf->arrayMO ["umem"],
									 $conf->arrayMO ["ufree"], $conf->arrayMO ["tmem"],
									 $conf->arrayMO ["util"]);
				
				break;
			case "umem":
				array_multisort($conf->arrayMO ["umem"], SORT_NUMERIC, $sortflag,
									 $conf->arrayMO ["node"], $conf->arrayMO ["status"],
									 $conf->arrayMO ["whois"], $conf->arrayMO ["cpus"],
									 $conf->arrayMO ["load"], $conf->arrayMO ["speed"],
									 $conf->arrayMO ["ufree"], $conf->arrayMO ["tmem"],
									 $conf->arrayMO ["util"]);
				
				break;
			case "ufree":
				array_multisort($conf->arrayMO ["ufree"], SORT_NUMERIC, $sortflag,
									 $conf->arrayMO ["node"], $conf->arrayMO ["status"],
									 $conf->arrayMO ["whois"], $conf->arrayMO ["cpus"],
									 $conf->arrayMO ["load"], $conf->arrayMO ["speed"],
									 $conf->arrayMO ["umem"], $conf->arrayMO ["tmem"],
									 $conf->arrayMO ["util"]);
				
				break;
			case "tmem":
				array_multisort($conf->arrayMO ["tmem"], SORT_NUMERIC, $sortflag,
									 $conf->arrayMO ["node"], $conf->arrayMO ["status"],
									 $conf->arrayMO ["whois"], $conf->arrayMO ["cpus"],
									 $conf->arrayMO ["load"], $conf->arrayMO ["speed"],
									 $conf->arrayMO ["umem"], $conf->arrayMO ["ufree"],
									 $conf->arrayMO ["util"]);
				
				break;
			case "util":
				array_multisort($conf->arrayMO ["util"], SORT_NUMERIC, $sortflag,
									 $conf->arrayMO ["node"], $conf->arrayMO ["status"],
									 $conf->arrayMO ["whois"], $conf->arrayMO ["cpus"],
									 $conf->arrayMO ["load"], $conf->arrayMO ["speed"],
									 $conf->arrayMO ["umem"], $conf->arrayMO ["ufree"],
									 $conf->arrayMO ["tmem"]);
				
				break;
			default:
				/* Do not order */
			} /* End Porn */
			
			foreach($conf->arrayMO["status"] as $none){
				$conf->web .= "<tr>";
				$conf->web .= show ($conf->arrayMO["node"][$i], "lr");
				if ($conf->arrayMO["status"][$i])
					$ok="ok";
				else
					$ok="nok";
			
				$conf->web .= show ("<center><img src=\"$ok.gif\" alt=\"\"></center>", "dr");
				$conf->web .= show ($conf->arrayMO["whois"][$i], "lr");
				$conf->web .= show ($conf->arrayMO["cpus"][$i], "dr");
				$conf->web .= show ($conf->arrayMO["load"][$i]/100, "lr");
				$conf->web .= show ($conf->arrayMO["speed"][$i], "dr");
				$conf->web .= show (intval($conf->arrayMO["umem"][$i] / $units) . "MB", "lr");
				$conf->web .= show (intval($conf->arrayMO["ufree"][$i] / $units) . "MB", "dr");
				$conf->web .= show (intval($conf->arrayMO["tmem"][$i] / $units) . "MB", "lr");
				$conf->web .= show ($conf->arrayMO["util"][$i] . "%", "dr");
				$conf->web .= "\n</tr>\n";
				$i++;
			}
		} else {
			/* The cluster is DOWN */
			$conf->web .= "<tr>";
			$conf->web .= show ("&nbsp;", "lr");
			$conf->web .= show ("&nbsp;", "dr");
			$conf->web .= show ("&nbsp;", "lr");
			$conf->web .= show ("&nbsp;", "dr");
			$conf->web .= show ("&nbsp;", "lr");
			$conf->web .= show ("&nbsp;", "dr");
			$conf->web .= show ("&nbsp;", "lr");
			$conf->web .= show ("&nbsp;", "dr");
			$conf->web .= show ("&nbsp;", "lr");
			$conf->web .= show ("&nbsp;", "dr");
			$conf->web .= "\n</tr>\n";
		}
		/* web tail */
		$conf->web .= "</table>\n";
		$conf->web .= "</center>\n";

		/* Show the web ! */
		echo $conf->web;

		if ($conf->doweb != "no"){
			include "down.inc";
		}
	}

	/* Shows the web for graph=procs */
	function procsplot($conf){
		global $units, $sort, $des, $csort, $order;
		$i=0;

		/* *********** */
		/* Set cookies */ 
		/* *********** */

		/* We use this cookie to set if we want
		 * "ascending order" or "descending order" */
		
		if ($conf->doweb != "no"){

			if ($order)
				$des=!$des;
			
			setcookie ("des", $des);

			if ($sort)
				setcookie ("csort", $sort);
			else
				$sort=$csort;

			/* web head */
			$conf->web .= webhead(10); /* 10 = refresh rate */
		}
		/* Table */
		$conf->web .= "<center>\n";
		$conf->web .= "<table class=\"pr\">\n";

		$conf->web .= "<tr>";
		$conf->web .= show (putlink($conf, "pid", "procs"), "lr");
		$conf->web .= show (putlink($conf, "n", "procs"), "dr");
		$conf->web .= show (putlink($conf, "lock", "procs"), "lr");
		$conf->web .= show (putlink($conf, "nmigs", "procs"), "dr");
		$conf->web .= show (putlink($conf, "stat", "procs"), "lr");
		$conf->web .= show (putlink($conf, "cmdline", "procs"), "dl");
		$conf->web .= show (putlink($conf, "nice", "procs"), "lr");
		$conf->web .= show (putlink($conf, "uid", "procs"), "dr");
		$conf->web .= show (putlink($conf, "cpu", "procs"), "lr");
		$conf->web .= "\n</tr>\n";

		/* Sort */
		if ($des)
			$sortflag = SORT_DESC;
		else
			$sortflag = SORT_ASC;

		switch ($sort){ /* Begin hardcore */
		case "pid":
			array_multisort($conf->arrayP ["pid"], SORT_NUMERIC, $sortflag,
								 $conf->arrayP ["n"], $conf->arrayP ["lock"],
								 $conf->arrayP ["nmigs"], $conf->arrayP ["stat"],
								 $conf->arrayP ["cmdline"], $conf->arrayP ["nice"],
								 $conf->arrayP ["uid"], $conf->arrayP ["cpu"]);
			break;
		case "n":
			array_multisort($conf->arrayP ["n"], SORT_NUMERIC, $sortflag,
								 $conf->arrayP ["pid"], $conf->arrayP ["lock"],
								 $conf->arrayP ["nmigs"], $conf->arrayP ["stat"],
								 $conf->arrayP ["cmdline"], $conf->arrayP ["nice"],
								 $conf->arrayP ["uid"], $conf->arrayP ["cpu"]);
			break;
		case "lock":
			array_multisort($conf->arrayP ["lock"], SORT_NUMERIC, $sortflag,
								 $conf->arrayP ["pid"], $conf->arrayP ["n"],
								 $conf->arrayP ["nmigs"], $conf->arrayP ["stat"],
								 $conf->arrayP ["cmdline"], $conf->arrayP ["nice"],
								 $conf->arrayP ["uid"], $conf->arrayP ["cpu"]);
			break;
		case "nmigs":
			array_multisort($conf->arrayP ["nmigs"], SORT_NUMERIC, $sortflag,
								 $conf->arrayP ["pid"], $conf->arrayP ["n"],
								 $conf->arrayP ["lock"], $conf->arrayP ["stat"],
								 $conf->arrayP ["cmdline"], $conf->arrayP ["nice"],
								 $conf->arrayP ["uid"], $conf->arrayP ["cpu"]);
			break;
		case "stat":
			array_multisort($conf->arrayP ["stat"], SORT_STRING, $sortflag,
								 $conf->arrayP ["pid"], $conf->arrayP ["n"],
								 $conf->arrayP ["lock"], $conf->arrayP ["nmigs"],
								 $conf->arrayP ["cmdline"], $conf->arrayP ["nice"],
								 $conf->arrayP ["uid"], $conf->arrayP ["cpu"]);
			break;
		case "cmdline":
			array_multisort($conf->arrayP ["cmdline"], SORT_STRING, $sortflag,
								 $conf->arrayP ["pid"], $conf->arrayP ["n"],
								 $conf->arrayP ["lock"], $conf->arrayP ["nmigs"],
								 $conf->arrayP ["stat"], $conf->arrayP ["nice"],
								 $conf->arrayP ["uid"], $conf->arrayP ["cpu"]);
			break;
		case "nice":
			array_multisort($conf->arrayP ["nice"], SORT_NUMERIC, $sortflag,
								 $conf->arrayP ["pid"], $conf->arrayP ["n"],
								 $conf->arrayP ["lock"], $conf->arrayP ["nmigs"],
								 $conf->arrayP ["stat"], $conf->arrayP ["cmdline"],
								 $conf->arrayP ["uid"], $conf->arrayP ["cmdline"]);
			break;
		case "uid":
			array_multisort($conf->arrayP ["uid"], SORT_NUMERIC, $sortflag,
								 $conf->arrayP ["pid"], $conf->arrayP ["n"],
								 $conf->arrayP ["lock"], $conf->arrayP ["nmigs"],
								 $conf->arrayP ["stat"], $conf->arrayP ["cmdline"],
								 $conf->arrayP ["nice"], $conf->arrayP ["cpu"]);
			break;
		case "cpu":
			array_multisort($conf->arrayP ["cpu"], SORT_NUMERIC, $sortflag,
								 $conf->arrayP ["pid"], $conf->arrayP ["n"],
								 $conf->arrayP ["lock"], $conf->arrayP ["nmigs"],
								 $conf->arrayP ["stat"], $conf->arrayP ["cmdline"],
								 $conf->arrayP ["nice"], $conf->arrayP ["uid"]);
			break;
		default:
			/* Do not order */
		} /* End Hardcore */

		if (!$conf->down){
			foreach($conf->arrayP["pid"] as $none){
				$conf->web .= "<tr>";
				$conf->web .= show ($conf->arrayP["pid"][$i], "lr");
				$conf->web .= show ($conf->arrayP["n"][$i], "dr");
				$conf->web .= show ($conf->arrayP["lock"][$i], "lr");
				$conf->web .= show ($conf->arrayP["nmigs"][$i], "dr");
				$conf->web .= show ($conf->arrayP["stat"][$i], "lr");
				$conf->web .= show ($conf->arrayP["cmdline"][$i], "dl");
				$conf->web .= show ($conf->arrayP["nice"][$i], "lr");
				$conf->web .= show ($conf->arrayP["uid"][$i], "dr");
				$c=sprintf("%.1f",$conf->arrayP["cpu"][$i]);
				$conf->web .= show ($c, "lr");
				$conf->web .= "\n</tr>\n";
				$i++;
			}
		} else {
			/* The cluster is DOWN */
			$conf->web .= "<tr>";
			$conf->web .= show ("&nbsp;", "lr");
			$conf->web .= show ("&nbsp;", "dr");
			$conf->web .= show ("&nbsp;", "lr");
			$conf->web .= show ("&nbsp;", "dr");
			$conf->web .= show ("&nbsp;", "lr");
			$conf->web .= show ("&nbsp;", "dr");
			$conf->web .= show ("&nbsp;", "lr");
			$conf->web .= show ("&nbsp;", "dr");
			$conf->web .= show ("&nbsp;", "lr");
			$conf->web .= "\n</tr>\n";
		}
		/* web tail */
		$conf->web .= "</table>\n";
		$conf->web .= "</center>\n";
		
		$conf->web .= "<a href=\"openmosixwebview.php?graph=remote\">View procs from remote</a>";

		/* Show the web ! */
		echo $conf->web;

		if ($conf->doweb != "no"){
			include "down.inc";
		}
	}

	/* Shows the web for graph=phist */
	function phistplot($conf){
		global $sort, $des, $csort, $x, $fg, $tm;
		$i=0;

		/* Returns an array with all the groups of the system */
		function groups (){
			global $group;
			$file=file($group);
			foreach ($file as $line){
				$g=explode(':',$line); /* root:x:0: */
				$arrayg[$g[0]]=$g[2];
			}
			return $arrayg;
		}

		/* *********** */
		/* Set cookies */ 
		/* *********** */

		/* We use this cookie to set if we want
		 * "ascending order" or "descending order" */
		if ($conf->doweb != "no"){
			if ($des)
				setcookie ("des", 0);
			else
				setcookie ("des", 1);
		
			if($conf->timestamp)
				setcookie("tm", $conf->timestamp);
			else
				$conf->timestamp=$tm;
		
			if ($x)
				setcookie("cx", $x);
			else
				$x=$cx;
		
			if ($sort)
				setcookie ("csort", $sort);
			else
				$sort=$csort;
		
			if ($conf->filtergroup){
				setcookie ("fg", $conf->filtergroup);
				$f=$conf->filtergroup;
			} else {
				if (isset($fg)) $f=$fg;
				else $f="all";
			}

			/* web head */
			$conf->web .= webhead();
		}
		$conf->web .= "<center>\n";
		$conf->web .= "<table class=\"pbig\">\n";
		$conf->web .= "<tr><td>\n";
		$conf->web .= "<center>\n";

		/* form */
		$conf->web .= "<form action=\"openmosixwebview.php?graph=phist\" method=\"post\">";
		$conf->web .= "time:";
		$conf->web .= "&nbsp;";
		$conf->web .= "<input type=\"text\" name=\"time\" size=\"20\" maxlength=\"20\" value=\"$conf->timestamp\" />";
		$conf->web .= "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;";
		$conf->web .= "<select name=\"filtergroup\" size=\"1\">\n";
		if ($f == "all")
			$conf->web .= "<option selected>all";
		else
			$conf->web .= "<option>all";

		$ag=groups(); /* get group's array */
		foreach ($ag as $var => $value){
			if ($var==$f)
				$conf->web .= "<option selected>$var";
			else
				$conf->web .= "<option>$var";
		}
		$conf->web .= "</select>\n";
		$conf->web .= "&nbsp;procs&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;";
		$conf->web .= "<input type=\"submit\" value=\"GO !\" />\n";
		$conf->web .= "</form>\n";
		$conf->web .= "<a href=\"openmosixwebview.php\"><img src=\"bar.php?i=$x\" border=\"0\" ismap alt=\"\" ></a>\n";
		/* Table */
		$conf->web .= "<table class=\"p\">\n";
		$conf->web .= "<tr>";
		$conf->web .= show (putlink($conf, "pid", "phist"), "lr"); 
		$conf->web .= show (putlink($conf, "uid", "phist"), "dr");
		$conf->web .= show (putlink($conf, "nice", "phist"), "lr");
		$conf->web .= show (putlink($conf, "n", "phist"), "dr");
		$conf->web .= show (putlink($conf, "lock", "phist"), "lr");
		$conf->web .= show (putlink($conf, "nmigs", "phist"), "dr");
		$conf->web .= show (putlink($conf, "stat", "phist"), "lr");
		$conf->web .= show (putlink($conf, "cantmove", "phist"), "dl");
		$conf->web .= show (putlink($conf, "cmdline", "phist"), "ll");
		$conf->web .= "\n</tr>\n";

		/* Sort */
		if ($des)
			$sortflag = SORT_DESC;
		else
			$sortflag = SORT_ASC;

		switch ($sort){ /* Begin hardcore */
		case "pid":
			array_multisort($conf->arrayPH ["pid"], SORT_NUMERIC, $sortflag,
								 $conf->arrayPH ["n"], $conf->arrayPH ["lock"],
								 $conf->arrayPH ["nmigs"], $conf->arrayPH ["stat"],
								 $conf->arrayPH ["cantmove"], $conf->arrayPH ["nice"],
								 $conf->arrayPH ["uid"], $conf->arrayPH ["cmdline"]);
			break;
		case "n":
			array_multisort($conf->arrayPH ["n"], SORT_NUMERIC, $sortflag,
								 $conf->arrayPH ["pid"], $conf->arrayPH ["lock"],
								 $conf->arrayPH ["nmigs"], $conf->arrayPH ["stat"],
								 $conf->arrayPH ["cantmove"], $conf->arrayPH ["nice"],
								 $conf->arrayPH ["uid"], $conf->arrayPH ["cmdline"]);
			break;
		case "lock":
			array_multisort($conf->arrayPH ["lock"], SORT_NUMERIC, $sortflag,
								 $conf->arrayPH ["pid"], $conf->arrayPH ["n"],
								 $conf->arrayPH ["nmigs"], $conf->arrayPH ["stat"],
								 $conf->arrayPH ["cantmove"], $conf->arrayPH ["nice"],
								 $conf->arrayPH ["uid"], $conf->arrayPH ["cmdline"]);
			break;
		case "nmigs":
			array_multisort($conf->arrayPH ["nmigs"], SORT_NUMERIC, $sortflag,
								 $conf->arrayPH ["pid"], $conf->arrayPH ["n"],
								 $conf->arrayPH ["lock"], $conf->arrayPH ["stat"],
								 $conf->arrayPH ["cantmove"], $conf->arrayPH ["nice"],
								 $conf->arrayPH ["uid"], $conf->arrayPH ["cmdline"]);
			break;
		case "stat":
			array_multisort($conf->arrayPH ["stat"], SORT_STRING, $sortflag,
								 $conf->arrayPH ["pid"], $conf->arrayPH ["n"],
								 $conf->arrayPH ["lock"], $conf->arrayPH ["nmigs"],
								 $conf->arrayPH ["cantmove"], $conf->arrayPH ["nice"],
								 $conf->arrayPH ["uid"], $conf->arrayPH ["cmdline"]);
			break;
		case "cantmove":
			array_multisort($conf->arrayPH ["cantmove"], SORT_STRING, $sortflag,
								 $conf->arrayPH ["pid"], $conf->arrayPH ["n"],
								 $conf->arrayPH ["lock"], $conf->arrayPH ["nmigs"],
								 $conf->arrayPH ["stat"], $conf->arrayPH ["nice"],
								 $conf->arrayPH ["uid"], $conf->arrayPH ["cmdline"]);
			break;
		case "nice":
			array_multisort($conf->arrayPH ["nice"], SORT_NUMERIC, $sortflag,
								 $conf->arrayPH ["pid"], $conf->arrayPH ["n"],
								 $conf->arrayPH ["lock"], $conf->arrayPH ["nmigs"],
								 $conf->arrayPH ["stat"], $conf->arrayPH ["cantmove"],
								 $conf->arrayPH ["uid"], $conf->arrayPH ["cmdline"]);
			break;
		case "uid":
			array_multisort($conf->arrayPH ["uid"], SORT_NUMERIC, $sortflag,
								 $conf->arrayPH ["pid"], $conf->arrayPH ["n"],
								 $conf->arrayPH ["lock"], $conf->arrayPH ["nmigs"],
								 $conf->arrayPH ["stat"], $conf->arrayPH ["cantmove"],
								 $conf->arrayPH ["nice"], $conf->arrayPH ["cmdline"]);
			break;
		case "cmdline":
			array_multisort($conf->arrayPH ["cmdline"], SORT_STRING, $sortflag,
								 $conf->arrayPH ["pid"], $conf->arrayPH ["n"],
								 $conf->arrayPH ["lock"], $conf->arrayPH ["nmigs"],
								 $conf->arrayPH ["stat"], $conf->arrayPH ["cantmove"],
								 $conf->arrayPH ["nice"], $conf->arrayPH ["uid"]);
			break;
		default:
			/* Do not order */
		} /* End Hardcore */
		if (!$conf->down){
			foreach($conf->arrayPH["pid"] as $none){
				if (($f == "all") || 
					 ($ag[$f] == $conf->arrayPH["uid"][$i])){   /* group filter */
					$conf->web .= "<tr>";
					$conf->web .= show ($conf->arrayPH ["pid"][$i], "lr");
					$conf->web .= show ($conf->arrayPH ["uid"][$i], "dr");
					$conf->web .= show ($conf->arrayPH ["nice"][$i], "lr");
					$conf->web .= show ($conf->arrayPH ["n"][$i], "dr");
					$conf->web .= show ($conf->arrayPH ["lock"][$i], "lr");
					$conf->web .= show ($conf->arrayPH ["nmigs"][$i], "dr");
					$conf->web .= show ($conf->arrayPH ["stat"][$i], "lr");
					$l=explode(",",$conf->arrayPH ["cantmove"][$i]);
					if ($l[0]=="M") $cantmove="";
					else {
						$cantmove= "";
						for ($j=0; $j<count($l); $j++){
							$cantmove .= substr($l[$j], 0, 4);
							if ($j!=count($l)-1) $cantmove .= ",";
						}
					}
					$conf->web .= show ($cantmove, "dl");
					$conf->web .= show ($conf->arrayPH ["cmdline"][$i], "ll");
					$conf->web .= "\n</tr>\n";
				}
				$i++;    
			}
		} else {
			$conf->web .= "<tr>";
			$conf->web .= show ("&nbsp;", "lr");
			$conf->web .= show ("&nbsp;", "dr");
			$conf->web .= show ("&nbsp;", "lr");
			$conf->web .= show ("&nbsp;", "dr");
			$conf->web .= show ("&nbsp;", "lr");
			$conf->web .= show ("&nbsp;", "dr");
			$conf->web .= show ("&nbsp;", "lr");
			$conf->web .= show ("&nbsp;", "dr");
			$conf->web .= show ("&nbsp;", "lr");
			$conf->web .= "\n</tr>\n";
		}
  
		/* web tail */
		$conf->web .= "</table>\n";
		$conf->web .= "</center>\n";
		$conf->web .= "</td></tr>\n";
		$conf->web .= "</table>\n";
		$conf->web .= "<table class=\"ptail\">\n";
		$conf->web .= "<tr>\n\t<td>\n";
		$conf->web .= "\t\t" . $conf->dirphist . $conf->selected . $conf->tsecsdat . "\n";
		$conf->web .= "\t</td>\n</tr>\n";
		$conf->web .= "</table>\n";
		$conf->web .= "</center>\n";

		/* Show the web ! */
		echo $conf->web;

		if ($conf->doweb != "no"){
			include "down.inc";
		}
	}

	function remoteplot($conf){
		global $units, $sort, $des, $csort, $order;
		$i=0;

		/* *********** */
		/* Set cookies */ 
		/* *********** */

		/* We use this cookie to set if we want
		 * "ascending order" or "descending order" */

		if ($conf->doweb != "no"){

			if ($order)
				$des=!$des;
			
			setcookie ("des", $des);

			if ($sort)
				setcookie ("csort", $sort);
			else
				$sort=$csort;

			/* web head */
			$conf->web .= webhead(20);
		}
		/* Table */
		$conf->web .= "<center>\n";
		$conf->web .= "<table class=\"re\">\n";

		$conf->web .= "<tr>";
		$conf->web .= show (putlink($conf, "node", "remote"), "lr");
		$conf->web .= show (putlink($conf, "ip", "remote"), "dr");
		$conf->web .= show (putlink($conf, "pid", "remote"), "lr");
		$conf->web .= "\n</tr>\n";

		if ($conf->arrayREMOTE["ip"][0]){ // got one remote
			/* Sort */
			if ($des)
				$sortflag = SORT_DESC;
			else
				$sortflag = SORT_ASC;
			
			switch ($sort){ /* Begin hardcore */
			case "node":
				array_multisort($conf->arrayREMOTE ["node"], SORT_NUMERIC, $sortflag,
									 $conf->arrayREMOTE ["ip"], $conf->arrayREMOTE ["pid"]);
				break;
			case "ip":
				array_multisort($conf->arrayREMOTE ["ip"], SORT_STRING, $sortflag,
									 $conf->arrayREMOTE ["node"], $conf->arrayREMOTE ["pid"]);
				break;
			case "pid":
				array_multisort($conf->arrayREMOTE ["pid"], SORT_NUMERIC, $sortflag,
									 $conf->arrayREMOTE ["node"], $conf->arrayREMOTE ["ip"]);
				break;
			default:
				/* Do not order */
			} /* End Hardcore */
		}
		if (!($conf->down) && $conf->arrayREMOTE["ip"][0]){
			foreach($conf->arrayREMOTE["ip"] as $none){
				$conf->web .= "<tr>";
				$conf->web .= show ($conf->arrayREMOTE["node"][$i], "lr");
				$conf->web .= show ($conf->arrayREMOTE["ip"][$i], "dr");
				$conf->web .= show ($conf->arrayREMOTE["pid"][$i], "lr");
				$conf->web .= "\n</tr>\n";
				$i++;
			}
		} else {
			/* The cluster is DOWN */
			$conf->web .= "<tr>";
			$conf->web .= show ("&nbsp;", "lr");
			$conf->web .= show ("&nbsp;", "dr");
			$conf->web .= show ("&nbsp;", "lr");
			$conf->web .= "\n</tr>\n";
		}
		/* web tail */
		$conf->web .= "</table>\n";
		$conf->web .= "</center>\n";

		/* Show the web ! */
		echo $conf->web;
		
		if ($conf->doweb != "no"){
			include "down.inc";
		}
	}

	/**
	 * We have the problem "redeclare of functions" in index2.php with:
	 * 
	 * include ("openmosixwebview.php");  
	 * include ("openmosixwebview.php");  
	 */
} /* end if (!inc) */

/*
 * $conf->width
 *        height
 *        file
 *        n
 *        title
 *        redarray
 *        redlegend 
 *        bluearray
 *        bluelegend
 *        ncpus
 *        maxload
 *        maxutil
 *        maxspeed
 */

if ($width)
     $conf->width=$width;
else
     $conf->width=$defaultwidth;
     
if ($height)
     $conf->height=$height;
else
     $conf->height=$defaultheight;

if ($n)
     $conf->n=$n;
else
     $conf->n=$defaultnode;

/* When the bar is pressed, we only get 160,7 */
/* Need to know if we are in that case         */
if ($QUERY_STRING){
	sscanf($QUERY_STRING, "%d,%d", &$x, &$y);
	if (isset($x) && isset($y)){
		$barorder=1;
		$graph="phist";
	}
}

if (substr_count($QUERY_STRING, "sort")){
	$order=1;
}

	/*
	 * arrayL -> Array for cluster's load
	 * arrayB -> Array for cluster's balancing efficiency
	 * arrayM -> Array for cluster's available memory
	 * arrayU -> Array for cluster's used memory
	 * arrayC -> Array for cluster's cpus
	 * arrayS -> Array for node's speed
	 * arrayL -> Array for node's load
	 * arrayPH -> Array for process history
	 * arrayUM -> Array for used memory (in MB)
	 * arrayMO -> Array with the info for graph=monitor
	 * arrayREMOTE -> Arry with the remote process
	 */

/* Available graph values */
$available = Array ("balancing", "available", "node", "load", "mem",
						  "speed", "util", "monitor", "procs", "phist", "remote");

if (!array_search($graph, $available)){
	$graph="balancing";
}

$conf->graph=$graph;

/* choose graph to plot */
switch ($graph){
 case "balancing":
	 /* Load - Balancing efficiency */ 
	 $conf = parsecollector($conf);
	 $conf->title= $host . " openMosix Cluster";

	 $conf->bluearray = $conf->arrayB;
	 $conf->bluelegend="load-balancing efficiency";

	 $conf->redarray = $conf->arrayL;
	 $conf->redlegend="overall load";
	 balplot($conf);
	 break;
 case "available":
	 /* Memory usage */ 
	 $conf = parsecollector($conf);
	 $conf->title="Memory";

	 $conf->bluearray = $conf->arrayM;
	 $conf->bluelegend="available";

	 $conf->redarray = $conf->arrayUM;
	 $conf->redlegend="used";

	 avaplot($conf);
	 break;
 case "node":
	 /* Node load */ 
	 $conf = parsecollector($conf);
	 $conf->title="Node " . $conf->n;

	 $conf->redarray = $conf->arrayL;
	 $conf->redlegend="load";

	 $conf->bluearray = $conf->arrayU;
	 $conf->bluelegend="memory";

	 nodeplot($conf);
	 break;
 case "load":
	 /* Load of each node (mosmon) */ 
	 $conf->title="CPU nodes load";

	 if (count($conf->ci=cids())>0) /* Get cluster node's id */ 
		 $arrayL=nodesload($conf);	 /* Get each node's load */
	 else
		 $conf->down=1; /* The cluster is down */
	 
	 /* Store array */
	 $conf->redarray = $arrayL;
	 $conf->redlegend="load";

	 loadplot($conf);
	 break;
 case "mem":
	 /* Mem used for each node (mosmon m) */ 
	 $conf->title="CPU nodes memory";

	 if (count($conf->ci=cids())>0) /* Get cluster node's id */ 
		 $mem=nodesmem($conf); /* Get each node's mem */
	 else
		 $conf->down=1; /* The cluster is down */

	 /* Store array */
	 $conf->redarray = $mem->total;
	 $conf->redlegend="used";

	 $conf->bluearray = $mem->used;
	 $conf->bluelegend="available";

	 memplot($conf);
	 break;
 case "speed":
	 /* Speed of each node (mosmon "l") */ 
	 $conf->title="CPU nodes speed";

	 if (count($conf->ci=cids())>0) /* Get cluster node's id */ 
		 $arrayS=nodesspeed($conf);  /* Get each node's speed */
	 else
		 $conf->down=1; /* The cluster is down */

	 /* Store array */
	 $conf->redarray = $arrayS;
	 $conf->redlegend="speed";

	 speedplot($conf);
	 break;
 case "util":
	 /* Processor utilizability percentage for each node (mosmon u) */ 
	 $conf->title="CPU nodes utilizability";

	 if (count($conf->ci=cids())>0) /* Get cluster node's id */ 
		 $util=nodesutil($conf); /* Get each node's util */
	 else
		 $conf->down=1; /* The cluster is down */

	 /* Store array */
	 $conf->redarray = $util;
	 $conf->redlegend="utilizability";

	 utilplot($conf);
	 break;
 case "monitor":
	 /* nodes status */
	 if (count($conf->ci=cids())>0) /* Get cluster node's id */
		 $conf = monitor($conf);
	 else
		 $conf->down=1;

	 $conf->doweb = $doweb; /* if we wan't <html> blab bla </html> */
	 monitorplot($conf);
	 break;
 case "procs":
	 $conf->doweb = $doweb; /* if we wan't <html> blab bla </html> */
	 $conf=procs($conf);
	 procsplot($conf);

	 break;
 case "phist":
	 /* Process history */
	 if ($d=@opendir($dirphist)){
		 closedir($d);
		 $conf->barwidth=$barwidth;
		 /* cookies stuff */
		 if ($sort) $conf->time = $tm; /* User clicked on a link */
		 if ($time) $conf->time = $time; /* form time */
		 if ($filtergroup || $barorder) $des=!$des;
		 
		 $conf->dirphist = $dirphist;
		 $conf->tsecsdat = $tsecsdat;
		 $conf->barorder = $barorder;
		 $conf->filtergroup = $filtergroup;
		 $conf = parsephist($conf);
	 } else {
		 $conf->down=1;
	 }
	 $conf->doweb = $doweb; /* if we wan't <html> blab bla </html> */
	 phistplot($conf);

	 break;
 case "remote":
	 $conf->doweb = $doweb; /* if we wan't <html> blab bla </html> */
	 $conf=remote($conf);
	 remoteplot($conf);

	 break;
}

unset ($conf);
return 1;
/* That's All Folks */
?>
